# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <math.h>
# include <vector>
using namespace std;
typedef vector<double> Data;

int	isPointIn(Data x,Data l,Data r)
{
	for(int i=0;i<x.size();i++) if(x[i]<l[i] || x[i]>r[i]) return 0;
	return 1;
}

double	problemFitness(Data x,Data l,Data r)
{
	if(!isPointIn(x,l,r)) return 1e+100;
	return x[0]*x[0]+x[1]*x[1]-cos(18.0*x[0])-cos(18.0*x[1]);
}

void	makeChromosome(Data &x,Data l,Data r)
{
	for(int i=0;i<x.size();i++)
		x[i]=l[i]+(r[i]-l[i])*drand48();
}

void	calculateFitness(vector<Data> genome,vector<double> &fitness,Data l,Data r)
{
	for(int i=0;i<genome.size();i++)
		fitness[i]=problemFitness(genome[i],l,r);
}

void	sortChromosomes(vector<Data> &genome,vector<double> &fitness)
{
	Data temp;
	temp.resize(genome[0].size());
	for(int i=0;i<genome.size();i++)
	{
		for(int j=0;j<genome.size()-1;j++)
		{
			if(fitness[j+1]<fitness[j])
			{
				double dtemp=fitness[j];
				fitness[j]=fitness[j+1];
				fitness[j+1]=dtemp;
				temp=genome[j];
				genome[j]=genome[j+1];
				genome[j+1]=temp;
			}
		}
	}
}

void	mutateChromosomes(vector<Data> &genome,Data l,Data r)
{
	double pm=1.0/l.size();//mutation rate
	for(int i=1;i<genome.size();i++)
	{
		for(int j=0;j<genome[i].size();j++)
		{
			double rt=drand48();
			if(rt<pm)
			{
				genome[i][j]=l[j]+(r[j]-l[j])*drand48();
			}
		}
	}
}


void	makeChild(Data &child,Data parent1,Data parent2)
{
	child.resize(parent1.size());
			
	//UNIFORM CROSSOVER
	for(int i=0;i<child.size();i++)
	{
		if(rand()%2==0) child[i]=parent1[i];
		else            child[i]=parent2[i];
	}
	
	/*	
	//DOUBLE CROSSOVER
	for(int i=0;i<child.size();i++)
	{
		double d=drand48();
		d=-0.25+1.50*d;
		child[i]=parent1[i]+d*(parent1[i]-parent2[i]);
	}
	*/
}

void	crossoverChromosomes(vector<Data> &genome,vector<double> fitness)
{
	const double pc=0.99;
	vector<Data> children;
	children.resize((int)(genome.size()*pc));
	const int tournament_size=4;
	int count_children=0;
	do
	{
		int parent[2];
		for(int i=0;i<2;i++)
		{
			int min_pos=0;
			double min_fitness=1e+100;
			for(int j=0;j<tournament_size;j++)
			{
				int r=rand() % genome.size();
				if(fitness[r]<min_fitness)
				{
					min_fitness=fitness[r];
					min_pos=r;
				}
			}
			parent[i]=min_pos;
		}
		makeChild(children[count_children],genome[parent[0]],genome[parent[1]]);
		count_children++;
		if(count_children==children.size()) break;
		makeChild(children[count_children],genome[parent[1]],genome[parent[0]]);
		count_children++;
	}while(count_children<children.size());
	//replace the worst chromosomes
	for(int i=0;i<count_children;i++)
	{
		genome[genome.size()-i-1]=children[i];
	}
}


int main()
{
	int problem_dimension;
	int genome_count;
	vector<Data> genome;
	int  maxgenerations;
	int  generation;
	Data	left,right;
	vector<double> fitness;
	int i;
	do
	{
		printf("PROBLEM DIMENSION? ");
		scanf("%d",&problem_dimension);
	}while(problem_dimension<=0);
	left.resize(problem_dimension);
	right.resize(problem_dimension);
	for(i=0;i<problem_dimension;i++)
	{
		do
		{
			printf("ENTER LEFT MARGIN %d ",i);
			scanf("%lf",&left[i]);
			printf("ENTER RIGHT MARGIN %d ",i);
			scanf("%lf",&right[i]);
		}while(right[i]<left[i]);
	}
	do
	{
		printf("NUMBER OF CHROMOSOMES? ");
		scanf("%d",&genome_count);
	}while(genome_count<=0);
	genome.resize(genome_count);
	for(i=0;i<genome_count;i++) 
	{
		genome[i].resize(problem_dimension);
		makeChromosome(genome[i],left,right);
	}
	fitness.resize(genome_count);
	do
	{
		printf("MAXIMUM NUMBER OF GENERATIONS ? ");
		scanf("%d",&maxgenerations);
	}while(maxgenerations<=0);

	double mean_best=0.0;
	double sx1=0.0,sx2=0.0;
	for(i=1;i<=maxgenerations;i++)
	{
		calculateFitness(genome,fitness,left,right);
		sortChromosomes(genome,fitness);
		mean_best+=fitness[0];
		sx1+=fabs(fitness[0]);
		sx2+=fabs(fitness[0])*fabs(fitness[0]);
		printf("THE BEST FITNESS AT GENERATION %d is %lf \n",i,fitness[0]);
		printf("DIFF BETWEEN MAX AND MIN %lf \n",fitness[genome_count-1]-fitness[0]);
		printf("MEAN OF BEST FITNESS %lf \n",mean_best/i);
		printf("STD  OF BEST FITNESS %lf \n",sx2/i-sx1/i*sx1/i);
		crossoverChromosomes(genome,fitness);
		mutateChromosomes(genome,left,right);
	}
	return 0;
}
